package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.config.SpringConfigs;
import com.xy.dto.MqttUserDto;
import com.xy.entity.MqttUser;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.MqttUserMapper;
import com.xy.service.factory.TaskCmdServiceImpl;
import com.xy.utils.*;
import com.xy.utils.enums.DictEnum;
import com.xy.utils.enums.DictSonEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;
import java.util.Map;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;


/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-02-17
 */
@Service
@AllArgsConstructor
@Api(tags = "mqtt认证")
public class MqttUserServiceImpl extends ServiceImpl<MqttUserMapper, MqttUser> implements MqttUserService {

    private RedisService<String> redisService;

    private SpringConfigs springConfigs;

    @PostMapping("obj")
    @ApiOperation("对象查询")
    public R<MqttUserDto.Vo> obj(@RequestBody MqttUserDto.Vo vo) {
        MqttUserDto.SelectList selectList = copy(MqttUserDto.SelectList.class, vo);
        List<MqttUserDto.Vo> list = list(selectList).getData();
        if (!Emptys.check(list)) {
            return R.ok();
        }
        MqttUserDto.Vo mqttUser = list.get(0);
        String env = getEnv(new MqttUserDto.Env().setDeviceId(mqttUser.getDeviceId())).getData();
        Map<String, SysDictRedis> map = SysDictUtils.get(DictEnum.ENV_PARAMS.getKey());
        SysDictRedis mqttUrl = map.get(env + "_mqtt_url");
        SysDictRedis serverUrl = map.get(env + "_server_url");
        mqttUser.setServerTime(System.currentTimeMillis())
                .setIpAddress(mqttUrl.getValue())
                .setServerUrl(serverUrl.getValue());
        return R.ok(mqttUser);
    }

    @PostMapping("getEnv")
    @ApiOperation("获取设备当前环境")
    public R<String> getEnv(@RequestBody @Validated MqttUserDto.Env envInfo) {
        String env = redisService.get(String.format(TaskCmdServiceImpl.deviceEnvKey, envInfo.getDeviceId()));
        if (!Emptys.check(env)) {
            SpringConfigs.Profiles profiles = springConfigs.getProfiles();
            env = profiles == null || profiles.getActive() == null ?
                    SysDictUtils.get(DictEnum.ENV.getKey(), DictSonEnum.ENV_UAT.getKey()).getValue() : profiles.getActive();
        }
        return R.ok(env);
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<MqttUserDto.Vo>> list(@RequestBody MqttUserDto.SelectList selectList) {
        LambdaQueryWrapper<MqttUser> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, MqttUser.class).build();
        List<MqttUser> list = list(lambdaQueryWrapper);
        return R.ok(copy(MqttUserDto.Vo.class, list));
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<MqttUserDto.Vo>> page(@RequestBody MqttUserDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<MqttUser> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, MqttUser.class).build();
        IPage<MqttUser> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(MqttUserDto.Vo.class, iPage));
    }

    @PostMapping("save")
    @ApiOperation("添加")
    public R save(@RequestBody @Validated MqttUserDto.Save save) {
        MqttUser saveInfo = copy(MqttUser.class, save);
        save(saveInfo);
        return R.ok();
    }
}